/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.lidar.amapvox.engine3d.entities.extended;

import com.jogamp.newt.event.InputEvent;
import com.jogamp.opengl.GL3bc;
import fr.amap.lidar.amapvox.engine3d.resources.materials.StandardMaterial;
import fr.amap.lidar.amapvox.engine3d.util.InputStatus;
import net.jcip.annotations.ThreadSafe;

/**
 * A particular component to draw a mesh in a scene.
 */
@ThreadSafe
public final class MeshRenderer extends Renderer {

    /**
     * Create a mesh renderer with a standard material and no mesh.
     */
    public MeshRenderer() {
        super();
        setMaterial(new StandardMaterial());
    }
    
    @Override
    public final void onInit(final GL3bc gl) {
        /* Nothing needed to do */
    }

    @Override
    public final void onInputting(final InputEvent event, final InputStatus status) {
        /* Nothing needed to do */
    }

    @Override
    public final void onUpdate(final GL3bc gl, final float delta) {
        if (getMaterial() == null) {
            throw(new NullPointerException("The current material is null."));
        }
        
        /* Send transform values to the material */
        getMaterial().setPosition(getParent().getTransform().getWorldPosition());
        getMaterial().setRotation(getParent().getTransform().getWorldRotation());
        getMaterial().setScale(getParent().getTransform().getWorldScale());
    }
    
    @Override
    public final void onDisplay(final GL3bc gl) {
        /* Nothing needed to do */
    }

    @Override
    public final void onRescale(final GL3bc gl, final int width, final int height) {
        /* Nothing needed to do */
    }

    @Override
    public final void onDestroy(final GL3bc gl) {
        /* Nothing needed to do */
    }

}
